/*
 *   $Id: anavor.c,v 1.6 2005/05/01 16:12:32 alexanto Exp $
 *
 *   BGL to XML converter
 *   Copyright (C)2004  Alessandro G. Antonini, Central Park Informatica
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Desc: VOR / ILS record analyzer $
 *
 *   $Log: anavor.c,v $
 *   Revision 1.6  2005/05/01 16:12:32  alexanto
 *   Changed loop code
 *
 *   Revision 1.5  2004/04/08 11:57:15  alexanto
 *   Made all unsigned types as DWORD
 *
 *   Revision 1.4  2004/02/25 13:47:04  alexanto
 *   Added description print with XML translation
 *
 *   Revision 1.3  2004/02/19 17:51:07  alexanto
 *   Some changes
 *
 *   Revision 1.2  2004/02/12 15:17:07  alexanto
 *   +Id+Log
 *   ,
 *
 *   Revision 1.1  2004/02/04 14:07:48  alexanto
 *   Initial revision
 *
 */
#include "bglxml.h"


/*
 * Analyzes and decode VORs
 *
 * Parameters:
 * nGrpCount - number of VOR available in this group
 * nGrpOffset - offset to current VOR chunk
 * nGrpLen - current chunk length
 *
 * Return value:
 * none
 */
void AnalyzeVor(DWORD nGrpCount, DWORD nGrpOffset, DWORD nChunkLen)
{
	DWORD i,nBytesRead = 0;
	WORD wType;				// navaid type - 0x13 for vor
	DWORD nLen;				// length of record in bytes
	BYTE bType;				// type
	BYTE bFlags;			// various flags - default 0x10, BIT_DME_ONLY or'ed
	int nLon;				// packed longitude
	int nLat;				// packed latitude
	int nAlt;				// altitude, meters * 1000
	int nFreq;				// navaid frequency * 1000
	float fRange;			// vor range
	float fMagVar;			// magnetic deviation
	DWORD nId;
	DWORD nRegion;
	char szId[4];			// encrypted navaid ID
	char szRegion[4];		// encrypted navaid Region
	WORD wDmeType;			// navaid type - 0x1600 for DME
	int nDmeLen;			// length of record in bytes
	WORD wUnk1;				// unknown word 1 - maybe used for 32 bits padding
	int nDmeLon;			// packed DME longitude
	int nDmeLat;			// packed DME latitude
	int nDmeAlt;			// DME altitude * 1000
	float fDmeRange;		// DME Range
	WORD wUnk2;				// unknown word 2
	int nOffset;			// relative pointer to index info ???
	char szDesc[64];		// navaid description
	
	char szLat[24];
	char szLon[24];
	char szAlt[24];
	char szRange[24];
	intlist_t *ptr;
	for (i = 0; i<nGrpCount; i++)
	{
		if (nBytesRead>nChunkLen)	// ad hoc
			break;
		
		if (!nTerse)
			PrintObjectSpec("VOR",nGrpOffset+nBytesRead);

		wType = GET_U16(nGrpOffset+nBytesRead);
		nLen = GET_U32(nGrpOffset+nBytesRead+2);
		bType = map.ptr[nGrpOffset+nBytesRead+6];
		nLon = GET_S32(nGrpOffset+nBytesRead+8);
		nLat = GET_S32(nGrpOffset+nBytesRead+12);
		nAlt = GET_S32(nGrpOffset+nBytesRead+16);
		if (nPartialDecode)		// if partial decode is enabled
		{
			if (gcdist(fCenterLat,fCenterLon,
				fslat2lat(nLat),fslon2lon(nLon))>fCenterRad)
			{	
				goto skip;	// skip the record if too far away
			}
		}
		
		if (bType==VOR_ILS)		// if we have an ILS just mark its offset
		{
			ptr = MALLOC(sizeof(intlist_t));
			PRECONDITION(NULL!=ptr);
			ptr->nOffset=(DWORD)(nGrpOffset+nBytesRead);
			ptr->next=NULL;
			if (NULL==lpIls)
			{
				lpIls=ptr;
				lpIlsTail=ptr;
			}
			else
			{
				lpIlsTail->next=ptr;
				lpIlsTail=ptr;
			}
			nBytesRead+=nLen;	// advance file pointer
		}
		else
		{
			bFlags = map.ptr[nGrpOffset+nBytesRead+7];
			if (!nTerse)
			{
				PrintObjectSpec("Vor",nGrpOffset+nBytesRead);
				fprintf(outfile,"<!-- VOR flags, 0x%.X-->\n", bFlags);
			}
			nFreq = GET_S32(nGrpOffset+nBytesRead+20);
			fRange = GET_FLOAT(nGrpOffset+nBytesRead+24);
			fMagVar = GET_FLOAT(nGrpOffset+nBytesRead+28);
			nId = GET_S32(nGrpOffset+nBytesRead+32);
			nRegion = GET_S32(nGrpOffset+nBytesRead+36);
			wUnk1 = GET_U16(nGrpOffset+nBytesRead+40);
			if (bFlags<0x10)	// e.g. this doesn't have a DME
			{
				nOffset = GET_S32(nGrpOffset+nBytesRead+42);
				memset(szDesc,0x0,sizeof(szDesc));
				memcpy(szDesc,&map.ptr[nGrpOffset+nBytesRead+46],nOffset-6);
				DecodeRegionStr(nRegion,szRegion);
			}
			else				// read additional info for DME
			{
				wDmeType = GET_U16(nGrpOffset+nBytesRead+40);
				nDmeLen = GET_S32(nGrpOffset+nBytesRead+42);
				nDmeLon = GET_S32(nGrpOffset+nBytesRead+48);
				nDmeLat = GET_S32(nGrpOffset+nBytesRead+52);
				nDmeAlt = GET_S32(nGrpOffset+nBytesRead+56);
				fDmeRange = GET_FLOAT(nGrpOffset+nBytesRead+60);
				wUnk2 = GET_U16(nGrpOffset+nBytesRead+64);
				nOffset = GET_S32(nGrpOffset+nBytesRead+66);
				memset(szDesc,0x0,sizeof(szDesc));
				memcpy(szDesc,OFFSET(nGrpOffset+nBytesRead+70),nOffset-6);
				DecodeRegionStr(nRegion,szRegion);
			}
			DecodeIdStr(nId,szId,0x42);

			fprintf(outfile,"\t<Vor lat=\"%s\" lon=\"%s\" alt=%s "
				"type=\"%s\"\n\t\tfrequency=\"%0.3lf\" magvar=\"%0.2lf\"",
				LatString(vorlat2lat(nLat),szLat),
				LatString(vorlon2lon(nLon),szLon), 
				AltString(nAlt,szAlt),
				vorTable[bType], (double)nFreq/1000000, MAGVAR(fMagVar));
			fprintf(outfile," range=%s region=\"%s\"\n\t\tident=\"%s\" name=",
				RangeString(fRange,szRange), szRegion, szId);
			
			PrintXmlString(szDesc,0);
			fprintf(outfile," ");
				

			switch (bFlags)		// gotta find something better
			{

			case 0x1:			// vor only
				fprintf(outfile,"dme=\"FALSE\" dmeOnly=\"FALSE\" >\n");
				break;
			
			case 0x10:			// dme only
				fprintf(outfile,"dme=\"TRUE\" dmeOnly=\"TRUE\" >\n");
				break;

			case 0x11:			// vor-dme
			case 0x31:			
				fprintf(outfile,"dme=\"TRUE\" dmeOnly=\"FALSE\" >\n");
				break;
			}

			if (bFlags>=0x10)
				fprintf(outfile,"\t\t<Dme lat=\"%s\" lon=\"%s\" "
					"alt=%s range=%s />\n",
					LatString(vorlat2lat(nDmeLat),szLat),
					LatString(vorlon2lon(nDmeLon),szLon),
					AltString(nDmeAlt,szAlt),
					RangeString(fDmeRange,szRange));
			fprintf(outfile, "\t</Vor>\n");
			fprintf(outfile, "\n");
skip:
			nBytesRead+=nLen;	// advance file pointer
		}
	}
}
