/*
 *   $Id: fueltab.c,v 1.5 2005/10/12 17:26:03 alexanto Exp $
 *
 *   BGL to XML converter
 *   Copyright (C)2004  Alessandro G. Antonini, Central Park Informatica
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Desc: Fuel-type lookup table $
 *
 *   $Log: fueltab.c,v $
 *   Revision 1.5  2005/10/12 17:26:03  alexanto
 *   adjusted fuel tab to match for fs fuel types (hopefully)
 *
 *   Revision 1.4  2005/05/01 16:12:32  alexanto
 *   Added get_fuel_kind routine
 *
 *   Revision 1.3  2004/04/08 11:57:15  alexanto
 *   Made all unsigned types as DWORD
 *
 *   Revision 1.2  2004/02/12 15:17:07  alexanto
 *   +Id+Log
 *
 *   Revision 1.1  2004/02/04 14:07:48  alexanto
 *   Initial revision
 *
 */
#include "bglxml.h"

/*fueltab_t fueltab[] =
{
	{ 0x40000000, "100",		FUELNO		},

	{ 0x40000001, "73",			FUELUNK		},
	{ 0x40000002, "73",			FUELPR		},
	{ 0x40000003, "73",			FUELYES		},

	{ 0x40000004, "87",			FUELUNK		},
	{ 0x40000008, "87",			FUELPR		},
	{ 0x4000000C, "87",			FUELYES		},

	{ 0x40000010, "100",		FUELUNK		},
	{ 0x40000020, "100",		FUELPR		},
	{ 0x40000030, "100",		FUELYES		},

	{ 0x40000040, "130",		FUELUNK		},
	{ 0x40000080, "130",		FUELPR		},
	{ 0x400000C0, "130",		FUELYES		},

	{ 0x40000100, "145",		FUELUNK		},
	{ 0x40000200, "145",		FUELPR		},
	{ 0x40000300, "145",		FUELYES		},

	{ 0x40004000, "MOGAS",		FUELUNK		},
	{ 0x40008000, "MOGAS",		FUELPR		},
	{ 0x4000C000, "MOGAS",		FUELYES		},

	{ 0x80000000, "JET",		FUELNO		},

	{ 0x80001000, "JET",		FUELUNK		},
	{ 0x80002000, "JET",		FUELPR		},
	{ 0x80003000, "JET",		FUELYES		},

	{ 0x80004000, "JETA",		FUELUNK		},
	{ 0x80008000, "JETA",		FUELPR		},
	{ 0x8000C000, "JETA",		FUELYES		},

	{ 0x80010000, "JETA1",		FUELUNK		},
	{ 0x80020000, "JETA1",		FUELPR		},
	{ 0x80030000, "JETA1",		FUELYES		},

	{ 0x80040000, "JETAP",		FUELUNK		},
	{ 0x80080000, "JETAP",		FUELPR		},
	{ 0x800C0000, "JETAP",		FUELYES		},

	{ 0x80100000, "JETB",		FUELUNK		},
	{ 0x80200000, "JETB",		FUELPR		},
	{ 0x80300000, "JETB",		FUELYES		},

	{ 0x80400000, "JET4",		FUELUNK		},
	{ 0x80800000, "JET4",		FUELPR		},
	{ 0x80C00000, "JET4",		FUELYES		},

	{ 0x81000000, "JET5",		FUELUNK		},
	{ 0x82000000, "JET5",		FUELPR		},
	{ 0x83000000, "JET5",		FUELYES		},

	{ 0x00000000, "JET",		FUELNO		}
};
*/
static const char *fuelkinds[] =
{"NO","UNKNOWN","PRIOR_REQUEST","YES"};

void get_fuel_kind(DWORD v, char *szkind, char *szavail)
{
	switch (v >> 28)						// test bit 30 or 31
    {
    case 0x4:								// fuel is piston type
        if ( (v & BIT10) || (v & BIT11))		// fuel type MOGAS
        {
            strcpy(szkind,"MOGAS");
            strcpy(szavail,fuelkinds[((v & BIT10) | (v & BIT11)) >> 10]);
        }
        else if ( (v & BIT8) || (v & BIT9))		// fuel type 160
        {
            strcpy(szkind,"145");
            strcpy(szavail,fuelkinds[((v & BIT8) | (v & BIT9)) >> 8]);
        }
        else if ( (v & BIT6) || (v & BIT7))		// fuel type 130
        {
            strcpy(szkind,"130");
            strcpy(szavail,fuelkinds[((v & BIT6) | (v & BIT7)) >> 6]);
        }
        else if ( (v & BIT4) || (v & BIT5))		// fuel type 100
        {
            strcpy(szkind,"100");
            strcpy(szavail,fuelkinds[((v & BIT4) | (v & BIT5)) >> 4]);
        }
        else if ( (v & BIT2) || (v & BIT3))		// fuel type 87
        {
            strcpy(szkind,"87");
            strcpy(szavail,fuelkinds[((v & BIT2) | (v & BIT3)) >> 2]);
        }
        else if ( (v & BIT0) || (v & BIT1))		// fuel type 73
        {
            strcpy(szkind,"73");
            strcpy(szavail,fuelkinds[(v & BIT0) | (v & BIT1)]);

        }
        else
        {
            strcpy(szkind,"73");				// we should never get here
            strcpy(szavail,fuelkinds[0]);
        }
        break;
	case 0x8:									// fuel find is jet
        if ( (v & BIT24) || (v & BIT25))		// fuel type JET5
        {
            strcpy(szkind,"JET5");
            strcpy(szavail,fuelkinds[((v & BIT24) | (v & BIT25)) >> 24]);
        }
        else if ( (v & BIT22) || (v & BIT23))	// fuel type JET4
        {
            strcpy(szkind,"JET4");
            strcpy(szavail,fuelkinds[((v & BIT22) | (v & BIT23)) >> 22]);
        }
        else if ( (v & BIT20) || (v & BIT21))	// fuel type JETB
        {
            strcpy(szkind,"JETB");
            strcpy(szavail,fuelkinds[((v & BIT20) | (v & BIT21)) >> 20]);
        }
        else if ( (v & BIT18) || (v & BIT19))	// fuel type JETAP
        {
            strcpy(szkind,"JETAP");
            strcpy(szavail,fuelkinds[((v & BIT18) | (v & BIT19)) >> 18]);
        }
        else if ( (v & BIT16) || (v & BIT17))	// fuel type JETA1
        {
            strcpy(szkind,"JETA1");
            strcpy(szavail,fuelkinds[((v & BIT16) | (v & BIT17)) >> 16]);
        }
        else if ( (v & BIT14) || (v & BIT15))	// fuel type JETA
        {
            strcpy(szkind,"JETA");
            strcpy(szavail,fuelkinds[((v & BIT14) | (v & BIT15)) >> 14]);
        }
        else if ( (v & BIT12) || (v & BIT13))	// fuel type JET
        {
            strcpy(szkind,"JET");
            strcpy(szavail,fuelkinds[((v & BIT12) | (v & BIT13)) >> 12]);
        }
        else
        {
            strcpy(szkind,"JET");
            strcpy(szavail,fuelkinds[0]);
        }
    	break;
	default:
		strcpy(szkind,"73");
		strcpy(szavail,fuelkinds[0]);
    }

}
